body.table {
  display: none;
}

body.table.lazy-facets-ready {
  display: block;
}

div.dropdown-menu li:has(.dropdown-facet) {
  /* we're taking over the facet options */
  display: none !important;
}

a:hover,
a:focus,
a:active  {
  color: #07407B;
  text-decoration: underline;
  text-decoration-thickness: 2px;
}

.facet-results a:visited {
  color: #276890;
}

ul.tight-bullets li {
  list-style-type: initial;
  white-space: nowrap;
}

ul.tight-bullets li a {
  white-space: normal;
}

.facet-info-name {
  text-align: center;
  font-family: Helvetica, sans-serif;
  font-size: 1.2em;
  letter-spacing: 1px;
}

.facet-results {
  justify-content: space-around;
}

.facet-info {
  width: auto;
  border: 0px solid #8872;
  padding: 0;
}

.facet-info li,
.facet-info ul {
  padding-bottom: 3px;
  padding-top: 3px;
}

.facet-info li {
  border-bottom: 1px solid #8872;
  display: flex;
  justify-content: space-between;
  gap: 1em;
}

.facet-info li:hover {
  background: #8872;
}
.facet-table-wrapper {
  display: flex;
}

.facet-results {
  position: sticky;
  max-height: 100vh;
  overflow: auto;
  display: block;
  align-self: flex-start;
  top: 0;
  min-width: 20em;
  justify-content: flex-start;
  flex-direction: column;
  margin-right: 25px;
}

.facet-info-name {
  position: sticky;
  top: 0;
  background: white;
}

@media only screen and (max-width: 576px) {
  .facet-table-wrapper {
    display: block;
  }

  .facet-results {
    position: static;
    max-height: initial;
  }
}


/* a CSS spinner, from https://stephanwagner.me/only-css-loading-spinner
 * use: <div class='spinner-container'><div class="spinner"></div></div>
 * */
@keyframes spinner {
  to {transform: rotate(360deg);}
}


.facets-loaded .spinner-container {
  display: none;
}

.spinner-container {
  position: relative;
  height: 40px;
}
 
.spinner:before {
  content: '';
  box-sizing: border-box;
  position: absolute;
  top: 50%;
  left: 50%;
  width: 20px;
  height: 20px;
  margin-top: -10px;
  margin-left: -10px;
  border-radius: 50%;
  border: 2px solid #ccc;
  border-top-color: #000;
  animation: spinner .6s linear infinite;
}
